use lru_cache::LruCache;
use hyper::client::Client;
use select::document::Document;
use select::predicate::Class;

use std::io::Read;

use {Error, Fighter, parse_weapons};

pub struct Ninjas {
    cache: LruCache<String, Ninja>,
    client: Client
}

impl Ninjas {
    pub fn new () -> Ninjas {
        Ninjas::with_cache_size(30)
    }

    pub fn with_cache_size (cache_size: usize) -> Ninjas {
        Ninjas {
            cache: LruCache::new(cache_size),
            client: Client::new()
        }
    }

    pub fn get (&mut self, name: &str) -> Result<&Ninja, Error> {
        if self.cache.contains_key(name) {
            return Result::Ok(self.cache.get_mut(name).unwrap())
        }

        let ninja = self.get_by_name(name)?;
        self.cache.insert(String::from(name), ninja);
        Result::Ok(self.cache.get_mut(name).unwrap())
    }

    fn get_by_name (&self, name: &str) -> Result<Ninja, Error> {
        let mut contents = String::new();
        self.client.get(&format!("http://www.majcher.com/pvn/pvn.cgi?a=n&n1={}", name)).send()?.read_to_string(&mut contents)?;
        Ninja::from_html(&contents[..])
    }
}

#[derive(Debug)]
pub struct Ninja {
    pub name: String,
    pub sneakiness: u8,
    pub pajamas: u8,
    pub pointy_things: u8,
    pub weapons: Vec<String>
}

impl Ninja {
    pub fn from_html (contents: &str) -> Result<Ninja, Error> {
        let document = Document::from(contents);
        let full_name = document.find(Class("ninja-name")).first()
                                .ok_or_else(|| Error::Parse(String::from("could not find ninja-name element")))?.text();

        let mut ninja = Ninja {
            name: String::from(full_name),
            sneakiness: 0,
            pajamas: 0,
            pointy_things: 0,
            weapons: vec![]
        };

        for node in document.find(Class("ninja-skill")).iter() {
            let skill = node.text();
            let score = node.parent().ok_or_else(|| Error::Parse(String::from("could not get ninja-skill parent element")))?
                                     .find(Class("ninja-score")).first()
                                     .ok_or_else(|| Error::Parse(String::from("could not get ninja-score element")))?
                                     .text();

            match &skill[..] {
                "Sneakiness:" => { ninja.sneakiness = score.parse::<u8>()? }
                "Pajamas:" => { ninja.pajamas = score.parse::<u8>()? },
                "Pointy Things:" => { ninja.pointy_things = score.parse::<u8>()? },
                "Weapons:" => { ninja.weapons = parse_weapons(&score[..]) }
                _ => { }
            }
        }

        Result::Ok(ninja)
    }
}

impl Fighter for Ninja {
    fn power (&self) -> u8 {
        self.sneakiness + self.pajamas + self.pointy_things + self.weapons.len() as u8
    }
}
