use lru_cache::LruCache;
use hyper::client::Client;
use select::document::Document;
use select::predicate::Class;

use std::io::Read;

use {Error, Fighter, parse_weapons};

pub struct Pirates {
    cache: LruCache<String, Pirate>,
    client: Client
}

impl Pirates {
    pub fn new () -> Pirates {
        Pirates::with_cache_size(30)
    }

    pub fn with_cache_size (cache_size: usize) -> Pirates {
        Pirates {
            cache: LruCache::new(cache_size),
            client: Client::new()
        }
    }

    pub fn get (&mut self, name: &str) -> Result<&Pirate, Error> {
        if self.cache.contains_key(name) {
            return Result::Ok(self.cache.get_mut(name).unwrap())
        }

        let pirate = self.get_by_name(name)?;
        self.cache.insert(String::from(name), pirate);
        Result::Ok(self.cache.get_mut(name).unwrap())
    }

    fn get_by_name (&self, name: &str) -> Result<Pirate, Error> {
        let mut contents = String::new();
        self.client.get(&format!("http://www.majcher.com/pvn/pvn.cgi?a=p&n1={}", name)).send()?.read_to_string(&mut contents)?;
        Pirate::from_html(&contents[..])
    }
}

#[derive(Debug)]
pub struct Pirate {
    pub name: String,
    pub swashbuckling: u8,
    pub drunkenness: u8,
    pub booty: u8,
    pub weapons: Vec<String>
}

impl Pirate {
    pub fn from_html (contents: &str) -> Result<Pirate, Error> {
        let document = Document::from(contents);
        let full_name = document.find(Class("pirate-name")).first()
                                .ok_or_else(|| Error::Parse(String::from("could not find pirate-name element")))?.text();

        let mut pirate = Pirate {
            name: String::from(full_name),
            swashbuckling: 0,
            drunkenness: 0,
            booty: 0,
            weapons: vec![]
        };

        for node in document.find(Class("pirate-skill")).iter() {
            let skill = node.text();
            let score = node.parent().ok_or_else(|| Error::Parse(String::from("could not get pirate-skill parent element")))?
                                     .find(Class("pirate-score")).first()
                                     .ok_or_else(|| Error::Parse(String::from("could not get pirate-score element")))?
                                     .text();

            match &skill[..] {
                "Swashbuckling:" => { pirate.swashbuckling = score.parse::<u8>()? }
                "Drunkenness:" => { pirate.drunkenness = score.parse::<u8>()? },
                "Booty:" => { pirate.booty = score.parse::<u8>()? },
                "Weapons:" => { pirate.weapons = parse_weapons(&score[..]) }
                _ => { }
            }
        }

        Result::Ok(pirate)
    }
}

impl Fighter for Pirate {
    fn power (&self) -> u8 {
        self.swashbuckling + self.drunkenness + self.booty + self.weapons.len() as u8
    }
}
